@@ -1,6 +1,7 @@
+/* SPDX-License-Identifier: BSD-2-Clause */
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2018 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2019 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -109,7 +110,7 @@
 #define DHCPCD_DHCP6			(1ULL << 50)
 #define DHCPCD_IF_UP			(1ULL << 51)
 #define DHCPCD_INFORM6			(1ULL << 52)
-#define DHCPCD_RTM_PPID			(1ULL << 53)
+// unused				(1ULL << 53)
 #define DHCPCD_IPV6RA_AUTOCONF		(1ULL << 54)
 #define DHCPCD_ROUTER_HOST_ROUTE_WARNED	(1ULL << 55)
 #define DHCPCD_LASTLEASE_EXTEND		(1ULL << 56)
@@ -181,7 +182,7 @@ struct if_options {
 	struct in_addr req_addr;
 	struct in_addr req_mask;
 	struct in_addr req_brd;
-	struct rt_head routes;
+	rb_tree_t routes;
 	struct in6_addr req_addr6;
 	uint8_t req_prefix_len;
 	unsigned int mtu;
