@@ -659,8 +659,13 @@ ipv4_addaddr(struct interface *ifp, const struct in_addr *addr,
 
 	ia->mask = *mask;
 	ia->brd = *bcast;
+#ifdef IP_LIFETIME
 	ia->vltime = vltime;
 	ia->pltime = pltime;
+#else
+	UNUSED(vltime);
+	UNUSED(pltime);
+#endif
 	snprintf(ia->saddr, sizeof(ia->saddr), "%s/%d",
 	    inet_ntoa(*addr), inet_ntocidr(*mask));
 
@@ -746,16 +751,36 @@ ipv4_applyaddr(void *arg)
 		return;
 	}
 
-#if __linux__
-	/* If the netmask or broadcast is different, re-add the addresss */
 	ia = ipv4_iffindaddr(ifp, &lease->addr, NULL);
+	/* If the netmask or broadcast is different, re-add the addresss.
+	 * If IP addresses do not have lifetimes, there is a very real chance
+	 * that re-adding them will scrub the subnet route temporarily
+	 * which is a bad thing, so avoid it. */
 	if (ia != NULL &&
-	    (ia->mask.s_addr != lease->mask.s_addr ||
-	    ia->brd.s_addr != lease->brd.s_addr))
-		ipv4_deladdr(ia, 0);
+	    ia->mask.s_addr == lease->mask.s_addr &&
+	    ia->brd.s_addr == lease->brd.s_addr)
+	{
+#ifndef IP_LIFETIME
+		logdebugx("%s: IP address %s already exists",
+		    ifp->name, ia->saddr);
 #endif
+	} else {
+#ifdef __linux__
+		/* Linux does not change netmask/broadcast address
+		 * for re-added addresses, so we need to delete the old one
+		 * first. */
+		if (ia != NULL)
+			ipv4_deladdr(ia, 0);
+#endif
+#ifndef IP_LIFETIME
+		if (ipv4_daddaddr(ifp, lease) == -1 && errno != EEXIST)
+			return;
+#endif
+	}
+#ifdef IP_LIFETIME
 	if (ipv4_daddaddr(ifp, lease) == -1 && errno != EEXIST)
 		return;
+#endif
 
 	ia = ipv4_iffindaddr(ifp, &lease->addr, NULL);
 	if (ia == NULL) {
